@@ -18,6 +18,7 @@
 
 #include "attrib.h"
 #include "device.h"
+#include "internal.h"
 #include "misc.h"
 #include "path.h"
 #include "scsi.h"
@@ -128,7 +129,7 @@ static bool zfcp_lun_devid_from_entry(struct zfcp_lun_devid *id_ptr,
 				      struct udev_entry_node *entry)
 {
 	struct zfcp_lun_devid id;
-	char *copy = NULL, *s;
+	char *copy = NULL, *s, *e, *u;
 	int i;
 	bool rc = false;
 
@@ -182,6 +183,28 @@ static bool zfcp_lun_devid_from_entry(struct zfcp_lun_devid *id_ptr,
 		goto out;
 	}
 
+	/*ENV{zdev_var__0_0_1941_0x500507630510c1ae_0x402340d400000000}="1"*/
+	if (starts_with(entry->key, "ENV{zdev_")) {
+		copy = misc_strdup(entry->key);
+
+		/* Find ID start (last __) and end (last }). */
+		s = misc_strrstr(copy, "__");
+		e = strrchr(copy, '}');
+		if (!s || !e)
+			goto out;
+		*e = 0;
+		s += 2;
+		/* Convert variable name to ID format. */
+		for (i = 0, u = s; (u = strchr(u, '_')); i++, u++) {
+			if (i < 2)
+				*u = '.';
+			else
+				*u = ':';
+		}
+		rc = zfcp_lun_parse_devid(&id, s, err_ignore) == EXIT_OK ?
+			true : false;
+	}
+
 out:
 	free(copy);
 	if (rc)
@@ -211,11 +234,46 @@ static struct zfcp_lun_node *zfcp_lun_node_from_entry(
 	return node;
 }
 
+static void add_internal_setting_from_entry(struct udev_entry_node *entry,
+					    struct zfcp_lun_node *node)
+{
+	char *copy, *name, *end, *u;
+	struct attrib *a;
+
+	/*ENV{zdev_var__0_0_1941_0x500507630510c1ae_0x402340d400000000}="1"*/
+	copy = misc_strdup(entry->key);
+
+	/* Find attribute name start and end. */
+	name = strchr(copy, '{');
+	end = misc_strrstr(copy, "__");
+	if (!name || !end)
+		goto out;
+	*end = 0;
+	name++;
+
+	/* zdev_ => zdev: */
+	u = strchr(name, '_');
+	if (u)
+		*u = ':';
+
+	a = attrib_find(zfcp_lun_subtype.dev_attribs, name);
+	setting_list_apply_actual(node->fc_settings, a, name, entry->value);
+
+out:
+	free(copy);
+}
+
 static void add_fc_setting_from_entry(struct udev_entry_node *entry,
 				      struct zfcp_lun_node *node)
 {
 	char *copy, *s, *e;
 
+	/*ENV{zdev_var__0_0_1941_0x500507630510c1ae_0x402340d400000000}="1"*/
+	if (starts_with(entry->key, "ENV{zdev_") &&
+	    strcmp(entry->op, "=") == 0) {
+		add_internal_setting_from_entry(entry, node);
+		return;
+	}
 	/*ATTR{[ccw/0.0.1941]0x500507630510c1ae/0x402340d400000000/failed}="0"*/
 	if (!starts_with(entry->key, "ATTR{[ccw/"))
 		return;
@@ -369,14 +427,14 @@ static exit_code_t lun_cb(const char *path, const char *name, void *data)
 
 /* Add the IDs for all zfcp lun devices for which a configuration exists to
  * LIST. */
-void udev_zfcp_lun_add_device_ids(struct util_list *list)
+void udev_zfcp_lun_add_device_ids(struct util_list *list, bool autoconf)
 {
 	struct lun_cb_data cb_data;
 	char *path;
 
 	cb_data.prefix = misc_asprintf("%s-%s-", UDEV_PREFIX, ZFCP_LUN_NAME);
 	cb_data.list = list;
-	path = path_get_udev_rules();
+	path = path_get_udev_rules(autoconf);
 
 	if (util_path_is_dir(path))
 		path_for_each(path, lun_cb, &cb_data);
@@ -385,7 +443,9 @@ void udev_zfcp_lun_add_device_ids(struct util_list *list)
 	free(cb_data.prefix);
 }
 
-static char *get_zfcp_lun_path(const char *id)
+/* Return path to zfcp lun udev rule file containing configuration data for
+ * all LUNs of a zfcp device. */
+static char *get_zfcp_lun_path(const char *id, bool autoconf)
 {
 	char *copy, *e, *path;
 
@@ -393,12 +453,19 @@ static char *get_zfcp_lun_path(const char *id)
 	e = strchr(copy, ':');
 	if (e)
 		*e = 0;
-	path = path_get_udev_rule(ZFCP_LUN_NAME, copy);
+	path = path_get_udev_rule(ZFCP_LUN_NAME, copy, autoconf);
 	free(copy);
 
 	return path;
 }
 
+/* Return path to zfcp lun udev rule file containing configuration data for
+ * a single LUN. */
+static char *get_single_zfcp_lun_path(const char *id, bool autoconf)
+{
+	return path_get_udev_rule(ZFCP_LUN_NAME, id, autoconf);
+}
+
 /* Apply the settings found in NODE to STATE. */
 static void zfcp_lun_node_to_state(struct zfcp_lun_node *node,
 				   struct attrib **attribs,
@@ -428,16 +495,22 @@ static void zfcp_lun_node_to_state(struct zfcp_lun_node *node,
 }
 
 /* Read the persistent configuration of a zfcp lun from a udev rule. */
-exit_code_t udev_zfcp_lun_read_device(struct device *dev)
+exit_code_t udev_zfcp_lun_read_device(struct device *dev, bool autoconf)
 {
 	struct subtype *st = dev->subtype;
-	struct device_state *state = &dev->persistent;
+	struct device_state *state = autoconf ? &dev->autoconf :
+						&dev->persistent;
 	struct util_list *luns;
 	struct zfcp_lun_node *node;
 	exit_code_t rc = EXIT_OK;
 	char *path;
 
-	path = get_zfcp_lun_path(dev->id);
+	/* Check for single lun file first then try multi lun file. */
+	path = get_single_zfcp_lun_path(dev->id, autoconf);
+	if (!util_path_exists(path)) {
+		free(path);
+		path = get_zfcp_lun_path(dev->id, autoconf);
+	}
 
 	/* Get previous rule data. */
 	luns = zfcp_lun_node_list_new();
@@ -475,7 +548,7 @@ static struct zfcp_lun_node *state_to_zfcp_lun_node(struct zfcp_lun_devid *id,
 					s->value);
 			setting_list_add(node->scsi_settings, n);
 		} else {
-			n = setting_new(NULL, s->name, s->value);
+			n = setting_new(s->attrib, s->name, s->value);
 			setting_list_add(node->fc_settings, n);
 		}
 	}
@@ -565,9 +638,21 @@ static exit_code_t write_luns_rule(const char *path, struct util_list *list)
 			node->id.lun);
 
 		util_list_iterate(&node->fc_settings->list, s) {
-			fprintf(fd, "ATTR{[ccw/%s]0x%016" PRIx64 "/0x%016"
-				PRIx64 "/%s}=\"%s\"\n", hba_id, node->id.wwpn,
-				node->id.lun, s->name, s->value);
+			if ((s->attrib && s->attrib->internal) ||
+			    internal_by_name(s->name)) {
+				fprintf(fd, "ENV{zdev_%s__%x_%x_%04x_0x%016"
+					PRIx64 "_0x%016" PRIx64 "}=\"%s\"\n",
+					internal_get_name(s->name),
+					node->id.fcp_dev.cssid,
+					node->id.fcp_dev.ssid,
+					node->id.fcp_dev.devno, node->id.wwpn,
+					node->id.lun, s->value);
+			} else {
+				fprintf(fd, "ATTR{[ccw/%s]0x%016" PRIx64
+					"/0x%016" PRIx64 "/%s}=\"%s\"\n",
+					hba_id, node->id.wwpn, node->id.lun,
+					s->name, s->value);
+			}
 		}
 		last_node = node;
 	}
@@ -599,8 +684,10 @@ out:
 
 /* Update the udev rule file that configures the zfcp lun with the specified
  * ID. If @state is %NULL, remove the rule, otherwise create a rule that
- * applies the corresponding parameters. */
-static exit_code_t update_lun_rule(const char *id, struct device_state *state)
+ * applies the corresponding parameters. If @single is set, update a single
+ * lun rule file, otherwise update a multi lun rule file. */
+static exit_code_t update_lun_rule(const char *id, struct device_state *state,
+				   bool single, bool autoconf)
 {
 	struct zfcp_lun_devid devid;
 	struct util_list *luns;
@@ -612,7 +699,8 @@ static exit_code_t update_lun_rule(const char *id, struct device_state *state)
 	rc = zfcp_lun_parse_devid(&devid, id, err_delayed_print);
 	if (rc)
 		return rc;
-	path = get_zfcp_lun_path(id);
+	path = single ? get_single_zfcp_lun_path(id, autoconf) :
+			get_zfcp_lun_path(id, autoconf);
 
 	/* Get previous rule data. */
 	luns = zfcp_lun_node_list_new();
@@ -648,27 +736,55 @@ static exit_code_t update_lun_rule(const char *id, struct device_state *state)
 
 /* Write a udev-rule to configure the specified zfcp lun and associated
  * device state. */
-exit_code_t udev_zfcp_lun_write_device(struct device *dev)
+exit_code_t udev_zfcp_lun_write_device(struct device *dev, bool autoconf)
 {
-	return update_lun_rule(dev->id, &dev->persistent);
+	exit_code_t rc;
+	struct device_state *state = autoconf ? &dev->autoconf :
+						&dev->persistent;
+
+	rc = update_lun_rule(dev->id, state, true, autoconf);
+
+	/* We only want single lun rule files so remove any remaining
+	 * references in multi lun rule files. */
+	update_lun_rule(dev->id, NULL, false, autoconf);
+
+	return rc;
 }
 
 /* Remove the UDEV rule used to configure the zfcp lun with the specified ID. */
-exit_code_t udev_zfcp_lun_remove_rule(const char *id)
+exit_code_t udev_zfcp_lun_remove_rule(const char *id, bool autoconf)
 {
-	return update_lun_rule(id, NULL);
+	exit_code_t rc, rc2;
+
+	rc = update_lun_rule(id, NULL, true, autoconf);
+	rc2 = update_lun_rule(id, NULL, false, autoconf);
+
+	if (rc)
+		return rc;
+
+	return rc2;
 }
 
 /* Determine if a udev rule exists for configuring the specified zfcp lun. */
-bool udev_zfcp_lun_exists(const char *id)
+bool udev_zfcp_lun_exists(const char *id, bool autoconf)
 {
 	struct zfcp_lun_devid devid;
-	char *path, *rule, *pattern = NULL;
+	char *path, *rule = NULL, *pattern = NULL;
 	bool rc = false;
 
 	if (zfcp_lun_parse_devid(&devid, id, err_ignore) != EXIT_OK)
 		return false;
-	path = get_zfcp_lun_path(id);
+
+	/* Check for single lun rule file first. */
+	path = get_single_zfcp_lun_path(id, autoconf);
+	if (util_path_exists(path)) {
+		rc = true;
+		goto out;
+	}
+	free(path);
+
+	/* Check multi lun rule file next. */
+	path = get_zfcp_lun_path(id, autoconf);
 	rule = misc_read_text_file(path, 1, err_ignore);
 	if (!rule)
 		goto out;
